@@ -17,7 +17,7 @@ from vim_quest_cli.engine.pure_python.KeyPressBindingHelper import (
     ActionNotPossibleException,
 )
 from vim_quest_cli.engine.pure_python.bindings import bind_engine_actions
-from vim_quest_cli.engine.pure_python.engine_helper import KeysHelper
+from vim_quest_cli.engine.pure_python.engine_helper import KeysHelper, KeyMatchStatus
 from vim_quest_cli.interface.mapping import Keys
 
 
@@ -49,26 +49,42 @@ class PurePythonEngine(EngineInterface, KeyHandlerBinder):
         if not state.command:
             return state
 
+        if state.command[0] is None or state.command[0] == "":
+            return state.copy(command=state.command[1:])
+
         repeat, c = KeysHelper.split_repeat(state.command)
         c = list(c)
 
-        for keys, fct in self._binded_keys.items():
-            if KeysHelper.does_keys_match(keys, c):
-                action, leftover = KeysHelper.does_keys_match(keys, c)
-                f_state = state.copy(command=leftover)
-                f_event = KeyPressEvent(
-                    repeat=repeat,
-                    keys_press=action,
-                )
+        had_partial_match = False
 
-                try:
-                    return fct(f_event, f_state)
-                except ActionNotPossibleException as _:
-                    return f_state
+        for keys, fct in self._binded_keys.items():
+            key_match_res = KeysHelper.extract_keys_match(c, keys)
+            if key_match_res.key_match_status == KeyMatchStatus.NO_MATCH:
+                continue
+            if (
+                key_match_res.key_match_status
+                == KeyMatchStatus.KEY_PRESS_IS_PARTIAL_COMMAND
+            ):
+                had_partial_match = True
+                continue
+
+            f_state = state.copy(command=key_match_res.keys_left)
+            f_event = KeyPressEvent(
+                repeat=repeat,
+                keys_press=key_match_res.keys_matching,
+            )
+
+            try:
+                return fct(f_event, f_state)
+            except ActionNotPossibleException as _:
+                return f_state
+
+        # If the command we were typing don't go anywhere, remove the keys.
+        if not had_partial_match:
+            return state.copy(command=())
 
         # TODO: it's not specifically like that
-        if c and (
-                c[-1] == Keys.Escape or c[-1] == Keys.ControlC):
+        if c and (c[-1] == Keys.Escape or c[-1] == Keys.ControlC):
             return state.copy(command=())
 
         raise NoCommandLeftException()
