@@ -30,6 +30,8 @@ def input_to_keys(input: str):
 
     I guess we need both, multiprocess input and management of longer inputs when copy-pasting.
     """
+    if not input:
+        return []
     if input in ansi_escape_sequences.ANSI_SEQUENCES:
         return [ansi_escape_sequences.ANSI_SEQUENCES[input]]
     return [input]
